@@ -81,7 +81,7 @@ static SGBucket bucketIndexFromFileName(const std::string& fileName)
   ss >> index;
   if (ss.fail())
     return SGBucket();
-  
+
   return SGBucket(index);
 }
 
@@ -93,7 +93,7 @@ struct ReaderWriterSTG::_ModelBin {
         osg::ref_ptr<SGReaderWriterOptions> _options;
     };
     struct _ObjectStatic {
-        _ObjectStatic() : _agl(false), _proxy(false), _lon(0), _lat(0), _elev(0), _hdg(0), _pitch(0), _roll(0), _range(SG_OBJECT_RANGE) { }
+        _ObjectStatic() : _agl(false), _proxy(false), _lon(0), _lat(0), _elev(0), _hdg(0), _pitch(0), _roll(0), _range(SG_OBJECT_RANGE_ROUGH) { }
         std::string _errorLocation;
         std::string _token;
         std::string _name;
@@ -195,21 +195,24 @@ struct ReaderWriterSTG::_ModelBin {
                 signBuilder.addSign(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev), i->_hdg, i->_name, i->_size);
             if (signBuilder.getSignsGroup())
                 group->addChild(signBuilder.getSignsGroup());
-            
+
             return group.release();
         }
-        
+
         mt _seed;
         std::list<_ObjectStatic> _objectStaticList;
         std::list<_Sign> _signList;
-        
+
         /// The original options to use for this bunch of models
         osg::ref_ptr<SGReaderWriterOptions> _options;
         SGBucket _bucket;
     };
-    
+
     _ModelBin() :
-        _object_range(SG_OBJECT_RANGE),
+        _object_range_bare(SG_OBJECT_RANGE_BARE),
+        _object_range_rough(SG_OBJECT_RANGE_ROUGH),
+        _object_range_detailed(SG_OBJECT_RANGE_DETAILED),
+        _building_mesh_enabled(false),
         _foundBase(false)
     { }
 
@@ -222,14 +225,14 @@ struct ReaderWriterSTG::_ModelBin {
         SGPath path = filePath;
         path.append(".."); path.append(".."); path.append("..");
         sharedOptions->getDatabasePathList().push_back(path.local8BitStr());
-        
+
         // ensure Models directory synced via TerraSync is searched before the copy in
         // FG_ROOT, so that updated models can be used.
         std::string terrasync_root = options->getPluginStringData("SimGear::TERRASYNC_ROOT");
         if (!terrasync_root.empty()) {
             sharedOptions->getDatabasePathList().push_back(terrasync_root);
         }
-        
+
         std::string fg_root = options->getPluginStringData("SimGear::FG_ROOT");
         sharedOptions->getDatabasePathList().push_back(fg_root);
 
@@ -268,21 +271,27 @@ struct ReaderWriterSTG::_ModelBin {
     {
         SGVec3d start = SGVec3d::fromGeod(SGGeod::fromGeodM(geod, 10000));
         SGVec3d end = SGVec3d::fromGeod(SGGeod::fromGeodM(geod, -1000));
-        
+
         osg::ref_ptr<osgUtil::LineSegmentIntersector> intersector;
         intersector = new osgUtil::LineSegmentIntersector(toOsg(start), toOsg(end));
         osgUtil::IntersectionVisitor visitor(intersector.get());
         group.accept(visitor);
-        
+
         if (!intersector->containsIntersections())
             return 0;
-        
+
         SGVec3d cart = toSG(intersector->getFirstIntersection().getWorldIntersectPoint());
         return SGGeod::fromCart(cart).getElevationM();
     }
-    
+
     bool read(const std::string& absoluteFileName, const osgDB::Options* options)
     {
+				// Determine object ranges.  Mesh size of 2000mx2000m needs to be accounted for.
+				_object_range_bare = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_BARE").c_str());
+				_object_range_rough = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_ROUGH").c_str());
+				_object_range_detailed = 1414.0f + atof(options->getPluginStringData("SimGear::LOD_RANGE_DETAILED").c_str());
+				_building_mesh_enabled = (options->getPluginStringData("SimGear::RENDER_BUILDING_MESH") == "true");
+
         if (absoluteFileName.empty())
             return false;
 
@@ -291,7 +300,7 @@ struct ReaderWriterSTG::_ModelBin {
             return false;
 
         SG_LOG(SG_TERRAIN, SG_INFO, "Loading stg file " << absoluteFileName);
-    
+
         std::string filePath = osgDB::getFilePath(absoluteFileName);
 
         // Bucket provides a consistent seed
@@ -304,7 +313,7 @@ struct ReaderWriterSTG::_ModelBin {
         bool onlyAirports = options->getPluginStringData("SimGear::FG_ONLY_AIRPORTS") == "ON";
         // do only load terrain btg files
         bool onlyTerrain = options->getPluginStringData("SimGear::FG_ONLY_TERRAIN") == "ON";
-        
+
         while (!stream.eof()) {
             // read a line
             std::string line;
@@ -314,30 +323,24 @@ struct ReaderWriterSTG::_ModelBin {
             std::string::size_type hash_pos = line.find('#');
             if (hash_pos != std::string::npos)
                 line.resize(hash_pos);
-            
+
             // and process further
             std::stringstream in(line);
-            
+
             std::string token;
             in >> token;
-            
+
             // No comment
             if (token.empty())
                 continue;
-            
+
             // Then there is always a name
             std::string name;
             in >> name;
-            
+
             SGPath path = filePath;
             path.append(name);
 
-            // Determine an appropriate range for the object, which has some randomness
-            double range = _object_range;
-            double lrand = mt_rand(&seed);
-            if      (lrand < 0.1) range = range * 2.0;
-            else if (lrand < 0.4) range = range * 1.5;
-
             if (token == "OBJECT_BASE") {
                 // Load only once (first found)
                 SG_LOG( SG_TERRAIN, SG_BULK, "    " << token << " " << name );
@@ -350,7 +353,7 @@ struct ReaderWriterSTG::_ModelBin {
                     obj._options = staticOptions(filePath, options);
                     _objectList.push_back(obj);
                 }
-                
+
             } else if (token == "OBJECT") {
                 if (!onlyAirports || isAirportBtg(name)) {
                     _Object obj;
@@ -360,81 +363,109 @@ struct ReaderWriterSTG::_ModelBin {
                     obj._options = staticOptions(filePath, options);
                     _objectList.push_back(obj);
                 }
-                
-            } else {
-                // Always OK to load
+
+            } else if (!onlyTerrain) {
+                // Load non-terrain objects
+
+								// Determine an appropriate range for the object, which has some randomness
+								double range = _object_range_rough;
+								double lrand = mt_rand(&seed);
+								if      (lrand < 0.1) range = range * 2.0;
+								else if (lrand < 0.4) range = range * 1.5;
+
                 if (token == "OBJECT_STATIC" || token == "OBJECT_STATIC_AGL") {
-                    if (!onlyTerrain) {
-                        osg::ref_ptr<SGReaderWriterOptions> opt;
-                        opt = staticOptions(filePath, options);
-                        if (SGPath(name).lower_extension() == "ac")
-                            opt->setInstantiateEffects(true);
-                        else
-                            opt->setInstantiateEffects(false);
-                        _ObjectStatic obj;
-                        obj._errorLocation = absoluteFileName;
-                        obj._token = token;
-                        obj._name = name;
-                        obj._agl = (token == "OBJECT_STATIC_AGL");
-                        obj._proxy = true;
-                        in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
-                        obj._range = range;
-                        obj._options = opt;
-                        _objectStaticList.push_back(obj);
-                    }
-                        
+										osg::ref_ptr<SGReaderWriterOptions> opt;
+										opt = staticOptions(filePath, options);
+										if (SGPath(name).lower_extension() == "ac")
+												opt->setInstantiateEffects(true);
+										else
+												opt->setInstantiateEffects(false);
+										_ObjectStatic obj;
+										obj._errorLocation = absoluteFileName;
+										obj._token = token;
+										obj._name = name;
+										obj._agl = (token == "OBJECT_STATIC_AGL");
+										obj._proxy = true;
+										in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+										obj._range = range;
+										obj._options = opt;
+										_objectStaticList.push_back(obj);
                 } else if (token == "OBJECT_SHARED" || token == "OBJECT_SHARED_AGL") {
-                    if (!onlyTerrain) {
-                        osg::ref_ptr<SGReaderWriterOptions> opt;
-                        opt = sharedOptions(filePath, options);
-                        if (SGPath(name).lower_extension() == "ac")
-                            opt->setInstantiateEffects(true);
-                        else
-                            opt->setInstantiateEffects(false);
-                        _ObjectStatic obj;
-                        obj._errorLocation = absoluteFileName;
-                        obj._token = token;
-                        obj._name = name;
-                        obj._agl = (token == "OBJECT_SHARED_AGL");
-                        obj._proxy = false;
-                        in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
-                        obj._range = range;
-                        obj._options = opt;
-                        _objectStaticList.push_back(obj);
-                    }
-
+										osg::ref_ptr<SGReaderWriterOptions> opt;
+										opt = sharedOptions(filePath, options);
+										if (SGPath(name).lower_extension() == "ac")
+												opt->setInstantiateEffects(true);
+										else
+												opt->setInstantiateEffects(false);
+										_ObjectStatic obj;
+										obj._errorLocation = absoluteFileName;
+										obj._token = token;
+										obj._name = name;
+										obj._agl = (token == "OBJECT_SHARED_AGL");
+										obj._proxy = false;
+										in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+										obj._range = range;
+										obj._options = opt;
+										_objectStaticList.push_back(obj);
                 } else if (token == "OBJECT_SIGN" || token == "OBJECT_SIGN_AGL") {
-                    if (!onlyTerrain) {
-                        _Sign sign;
-                        sign._token = token;
-                        sign._name = name;
-                        sign._agl = (token == "OBJECT_SIGN_AGL");
-                        in >> sign._lon >> sign._lat >> sign._elev >> sign._hdg >> sign._size;
-                        _signList.push_back(sign);
-                    }
+										_Sign sign;
+										sign._token = token;
+										sign._name = name;
+										sign._agl = (token == "OBJECT_SIGN_AGL");
+										in >> sign._lon >> sign._lat >> sign._elev >> sign._hdg >> sign._size;
+										_signList.push_back(sign);
+                } else if (token == "OBJECT_BUILDING_MESH_ROUGH" || token == "OBJECT_BUILDING_MESH_DETAILED") {
 
+										// Only load if building mesh enabled to avoid impacting low-powered systems
+										if (_building_mesh_enabled) {
+											osg::ref_ptr<SGReaderWriterOptions> opt;
+											opt = staticOptions(filePath, options);
+											if (SGPath(name).lower_extension() == "ac")
+													opt->setInstantiateEffects(true);
+											else
+													opt->setInstantiateEffects(false);
+											_ObjectStatic obj;
+											obj._errorLocation = absoluteFileName;
+											obj._token = token;
+											obj._name = name;
+											obj._agl = false;
+											obj._proxy = true;
+											in >> obj._lon >> obj._lat >> obj._elev >> obj._hdg >> obj._pitch >> obj._roll;
+
+											if (token == "OBJECT_BUILDING_MESH_DETAILED") {
+												// Apply a lower LOD range if this is a detailed building
+												range = _object_range_detailed;
+												double lrand = mt_rand(&seed);
+												if      (lrand < 0.1) range = range * 2.0;
+												else if (lrand < 0.4) range = range * 1.5;
+											}
+
+											obj._range = range;
+											obj._options = opt;
+											_objectStaticList.push_back(obj);
+                    }
                 } else {
                     SG_LOG( SG_TERRAIN, SG_ALERT, absoluteFileName
                             << ": Unknown token '" << token << "'" );
                 }
+            } else {
+                SG_LOG( SG_TERRAIN, SG_ALERT, absoluteFileName
+                        << ": Unknown token '" << token << "'" );
             }
         }
-        
+
         return true;
     }
-    
+
     osg::Node* load(const SGBucket& bucket, const osgDB::Options* opt)
     {
         osg::ref_ptr<SGReaderWriterOptions> options;
         options = SGReaderWriterOptions::copyOrCreate(opt);
 
-        // Determine object ranges
-        _object_range = atof(options->getPluginStringData("SimGear::ROUGH_LOD_RANGE").c_str());
-
         osg::ref_ptr<osg::Group> terrainGroup = new osg::Group;
         terrainGroup->setDataVariance(osg::Object::STATIC);
         terrainGroup->setName("terrain");
-        
+
         if (_foundBase) {
             for (std::list<_Object>::iterator i = _objectList.begin(); i != _objectList.end(); ++i) {
                 osg::ref_ptr<osg::Node> node;
@@ -448,7 +479,7 @@ struct ReaderWriterSTG::_ModelBin {
             }
         } else {
             SG_LOG(SG_TERRAIN, SG_INFO, "  Generating ocean tile: " << bucket.gen_base_path() << "/" << bucket.gen_index_str());
-            
+
             osg::Node* node = SGOceanTile(bucket, options->getMaterialLib());
             if (node) {
                 node->setName("SGOceanTile");
@@ -464,7 +495,7 @@ struct ReaderWriterSTG::_ModelBin {
                 continue;
             i->_elev += elevation(*terrainGroup, SGGeod::fromDeg(i->_lon, i->_lat));
         }
-        
+
         for (std::list<_Sign>::iterator i = _signList.begin(); i != _signList.end(); ++i) {
             if (!i->_agl)
                 continue;
@@ -478,7 +509,7 @@ struct ReaderWriterSTG::_ModelBin {
             osg::PagedLOD* pagedLOD = new osg::PagedLOD;
             pagedLOD->setCenterMode(osg::PagedLOD::USE_BOUNDING_SPHERE_CENTER);
             pagedLOD->setName("pagedObjectLOD");
-            
+
             // This should be visible in any case.
             // If this is replaced by some lower level of detail, the parent LOD node handles this.
             pagedLOD->addChild(terrainGroup, 0, std::numeric_limits<float>::max());
@@ -492,17 +523,20 @@ struct ReaderWriterSTG::_ModelBin {
             osg::ref_ptr<osgDB::Options> callbackOptions = new osgDB::Options;
             callbackOptions->setReadFileCallback(readFileCallback.get());
             pagedLOD->setDatabaseOptions(callbackOptions.get());
-            
+
             pagedLOD->setFileName(pagedLOD->getNumChildren(), "Dummy name - use the stored data in the read file callback");
 
             // Objects may end up displayed up to 2x the object range.
-            pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 2.0 * _object_range + SG_TILE_RADIUS);
-            
+            pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 2.0 * _object_range_rough + SG_TILE_RADIUS);
+
             return pagedLOD;
         }
     }
-    
-    double _object_range;
+
+    double _object_range_bare;
+    double _object_range_rough;
+    double _object_range_detailed;
+    bool _building_mesh_enabled;
     bool _foundBase;
     std::list<_Object> _objectList;
     std::list<_ObjectStatic> _objectStaticList;
@@ -541,11 +575,11 @@ ReaderWriterSTG::readNode(const std::string& fileName, const osgDB::Options* opt
         // For stg meta files, we need options for the search path.
         if (!options)
             return ReadResult::FILE_NOT_FOUND;
-        
+
         SG_LOG(SG_TERRAIN, SG_INFO, "Loading tile " << fileName);
-        
+
         std::string basePath = bucket.gen_base_path();
-        
+
         // Stop scanning once an object base is found
         // This is considered a meta file, so apply the scenery path search
         const osgDB::FilePathList& filePathList = options->getDatabasePathList();
@@ -556,7 +590,7 @@ ReaderWriterSTG::readNode(const std::string& fileName, const osgDB::Options* opt
             objects.append(basePath);
             objects.append(fileName);
             modelBin.read(objects.local8BitStr(), options);
-            
+
             SGPath terrain(*i);
             terrain.append("Terrain");
             terrain.append(basePath);
